# AUTOGENERATED! DO NOT EDIT! File to edit: ../05_ConvertTimeSeriesResultProtoToPandas.ipynb.

# %% auto 0
__all__ = [
    "logger",
    "ch",
    "formatter",
    "convert_proto_timeseries_to_pd_series",
    "convert_proto_timeseries_to_pd_dataframe",
    "convert_proto_timeseries_for_multiple_propulsors_to_pd_dataframe",
]

# %% ../05_ConvertTimeSeriesResultProtoToPandas.ipynb 3
import pandas as pd
import numpy as np
import MachSysS.gymir_result_pb2 as proto
import logging


# Define logger
logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

ch = logging.StreamHandler()
ch.setLevel(logging.INFO)

formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
ch.setFormatter(formatter)

logger.addHandler(ch)


def convert_proto_timeseries_to_pd_series(
    proto_timeseries: proto.TimeSeriesResult,
) -> pd.Series:
    """Convert a TimeSeriesResult proto message to a pandas Series for propulsion power"""
    time = map(lambda each: each.epoch_s, proto_timeseries.propulsion_power_timeseries)
    propulsion_power = map(
        lambda each: each.propulsion_power_kw,
        proto_timeseries.propulsion_power_timeseries,
    )
    return pd.Series(index=time, data=propulsion_power)


def convert_proto_timeseries_to_pd_dataframe(
    proto_timeseries: proto.TimeSeriesResult,
) -> pd.DataFrame:
    """
    Convert a TimeSeriesResult proto message to a pandas DataFrame
    for propulsion and auxiliary power and other operational parameters such as speed and draft
    """
    time = list(
        map(lambda each: each.epoch_s, proto_timeseries.propulsion_power_timeseries)
    )
    propulsion_power = list(
        map(
            lambda each: each.propulsion_power_kw,
            proto_timeseries.propulsion_power_timeseries,
        )
    )
    auxiliary_power = list(
        map(
            lambda each: each.auxiliary_power_kw,
            proto_timeseries.propulsion_power_timeseries,
        )
    )
    if (np.array(auxiliary_power) == 0).all():
        auxiliary_power = [proto_timeseries.auxiliary_power_kw] * len(time)
    df = pd.DataFrame(
        index=time,
        data=dict(
            propulsion_power_kw=propulsion_power,
            auxiliary_power_kw=auxiliary_power,
        ),
    )
    if len(proto_timeseries.operation_profile) > 0:
        time_operation_profile = list(
            map(lambda each: each.epoch_s, proto_timeseries.operation_profile)
        )
        speed_array = np.array(
            list(map(lambda each: each.speed_kn, proto_timeseries.operation_profile))
        )
        draft_array = np.array(
            list(map(lambda each: each.draft_m, proto_timeseries.operation_profile))
        )

        if time_operation_profile != time:
            logger.warning(
                "Time in operation profile is not the same as in propulsion power."
                "The operation profile will be interpolated to match the propulsion power time."
            )
            time_array = np.array(time_operation_profile)
            time_array_ref = np.array(time)
            speed_array = np.interp(time_array_ref, time_array, speed_array)
            draft_array = np.interp(time_array_ref, time_array, draft_array)

        df["speed_kn"] = speed_array
        df["draft_m"] = draft_array
    return df


def convert_proto_timeseries_for_multiple_propulsors_to_pd_dataframe(
    proto_timeseries: proto.TimeSeriesResultForMultiplePropulsors,
) -> pd.DataFrame:
    """
    Convert a TimeSeriesResult proto message to a pandas DataFrame
    for propulsion and auxiliary power and other operational parameters such as speed and draft
    """
    time = list(
        map(lambda each: each.epoch_s, proto_timeseries.propulsion_power_timeseries)
    )
    propulsor_names = list(map(lambda each: each, proto_timeseries.propulsor_names))
    propulsion_power = np.array(
        list(
            map(
                lambda each_instance: list(
                    map(
                        lambda each_power: each_power, each_instance.propulsion_power_kw
                    )
                ),
                proto_timeseries.propulsion_power_timeseries,
            )
        )
    )
    assert propulsion_power.shape[0] == len(
        time
    ), "Time and propulsion power length mismatch"
    assert propulsion_power.shape[1] == len(
        propulsor_names
    ), "Propulsion power and propulsor names length mismatch"
    data = {
        propulsor_name: propulsion_power[:, i]
        for i, propulsor_name in enumerate(propulsor_names)
    }
    auxiliary_power = list(
        map(
            lambda each: each.auxiliary_power_kw,
            proto_timeseries.propulsion_power_timeseries,
        )
    )
    if (np.array(auxiliary_power) == 0).all():
        auxiliary_power = [proto_timeseries.auxiliary_power_kw] * len(time)
    data["auxiliary_power_kw"] = auxiliary_power
    df = pd.DataFrame(
        index=time,
        data=data,
    )
    if len(proto_timeseries.operation_profile) > 0:
        time_operation_profile = list(
            map(lambda each: each.epoch_s, proto_timeseries.operation_profile)
        )
        speed_array = np.array(
            list(map(lambda each: each.speed_kn, proto_timeseries.operation_profile))
        )
        draft_array = np.array(
            list(map(lambda each: each.draft_m, proto_timeseries.operation_profile))
        )

        if time_operation_profile != time:
            logger.warning(
                "Time in operation profile is not the same as in propulsion power."
                "The operation profile will be interpolated to match the propulsion power time."
            )
            time_array = np.array(time_operation_profile)
            time_array_ref = np.array(time)
            speed_array = np.interp(time_array_ref, time_array, speed_array)
            draft_array = np.interp(time_array_ref, time_array, draft_array)

        df["speed_kn"] = speed_array
        df["draft_m"] = draft_array
    return df



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: system_structure.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x16system_structure.proto\x12\x18machinerySystemStructure"\x1d\n\x05Point\x12\t\n\x01x\x18\x01 \x01(\x01\x12\t\n\x01y\x18\x02 \x01(\x01":\n\x07\x43urve1D\x12/\n\x06points\x18\x01 \x03(\x0b\x32\x1f.machinerySystemStructure.Point"_\n\tBSFCCurve\x12\x0f\n\x07x_label\x18\x01 \x01(\t\x12\x0f\n\x07y_label\x18\x02 \x01(\t\x12\x30\n\x05\x63urve\x18\x03 \x01(\x0b\x32!.machinerySystemStructure.Curve1D"e\n\x0f\x45\x66\x66iciencyCurve\x12\x0f\n\x07x_label\x18\x01 \x01(\t\x12\x0f\n\x07y_label\x18\x02 \x01(\t\x12\x30\n\x05\x63urve\x18\x03 \x01(\x0b\x32!.machinerySystemStructure.Curve1D"X\n\x04\x42SFC\x12\x32\n\x05\x63urve\x18\x01 \x01(\x0b\x32#.machinerySystemStructure.BSFCCurve\x12\x12\n\x05value\x18\x02 \x01(\x01H\x00\x88\x01\x01\x42\x08\n\x06_value"s\n\nEfficiency\x12=\n\x05\x63urve\x18\x01 \x01(\x0b\x32).machinerySystemStructure.EfficiencyCurveH\x00\x88\x01\x01\x12\x12\n\x05value\x18\x02 \x01(\x01H\x01\x88\x01\x01\x42\x08\n\x06_curveB\x08\n\x06_value"`\n\nPowerCurve\x12\x0f\n\x07x_label\x18\x01 \x01(\t\x12\x0f\n\x07y_label\x18\x02 \x01(\t\x12\x30\n\x05\x63urve\x18\x03 \x01(\x0b\x32!.machinerySystemStructure.Curve1D"\x85\x01\n\x19PropulsionPowerTimeSeries\x12\x0f\n\x07x_label\x18\x01 \x01(\t\x12\x0f\n\x07y_label\x18\x02 \x01(\t\x12\x14\n\x0cpropulsor_id\x18\x03 \x01(\r\x12\x30\n\x05\x63urve\x18\x04 \x01(\x0b\x32!.machinerySystemStructure.Curve1D"\x85\x01\n\x17\x41uxiliaryLoadTimeSeries\x12\x0f\n\x07x_label\x18\x01 \x01(\t\x12\x0f\n\x07y_label\x18\x02 \x01(\t\x12\x16\n\x0eswitchboard_id\x18\x03 \x01(\r\x12\x30\n\x05\x63urve\x18\x04 \x01(\x0b\x32!.machinerySystemStructure.Curve1D"8\n\rAuxiliaryLoad\x12\x16\n\x0eswitchboard_id\x18\x01 \x01(\r\x12\x0f\n\x07load_kw\x18\x02 \x01(\x01"\xa2\x01\n\rEmissionCurve\x12\x0f\n\x07x_label\x18\x01 \x01(\t\x12\x0f\n\x07y_label\x18\x02 \x01(\t\x12\x30\n\x05\x63urve\x18\x03 \x01(\x0b\x32!.machinerySystemStructure.Curve1D\x12=\n\remission_type\x18\x04 \x01(\x0e\x32&.machinerySystemStructure.EmissionType"\xe5\x01\n\x04Gear\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x12\n\ngear_ratio\x18\x02 \x01(\x01\x12\x16\n\x0erated_power_kw\x18\x03 \x01(\x01\x12\x17\n\x0frated_speed_rpm\x18\x04 \x01(\x01\x12\x38\n\nefficiency\x18\x05 \x01(\x0b\x32$.machinerySystemStructure.Efficiency\x12+\n#order_from_switchboard_or_shaftline\x18\x06 \x01(\r\x12\x16\n\x0eunit_price_usd\x18\x07 \x01(\x01\x12\x0b\n\x03uid\x18\x08 \x01(\t"x\n\x04\x46uel\x12\x35\n\tfuel_type\x18\x01 \x01(\x0e\x32".machinerySystemStructure.FuelType\x12\x39\n\x0b\x66uel_origin\x18\x02 \x01(\x0e\x32$.machinerySystemStructure.FuelOrigin"\x92\x06\n\x06\x45ngine\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x16\n\x0erated_power_kw\x18\x02 \x01(\x01\x12\x17\n\x0frated_speed_rpm\x18\x03 \x01(\x01\x12,\n\x04\x62sfc\x18\x04 \x01(\x0b\x32\x1e.machinerySystemStructure.BSFC\x12\x31\n\tmain_fuel\x18\x05 \x01(\x0b\x32\x1e.machinerySystemStructure.Fuel\x12+\n#order_from_switchboard_or_shaftline\x18\x06 \x01(\r\x12\x32\n\npilot_bsfc\x18\x07 \x01(\x0b\x32\x1e.machinerySystemStructure.BSFC\x12\x32\n\npilot_fuel\x18\x08 \x01(\x0b\x32\x1e.machinerySystemStructure.Fuel\x12U\n\x16nox_calculation_method\x18\t \x01(\x0e\x32\x35.machinerySystemStructure.Engine.NOxCalculationMethod\x12@\n\x0f\x65mission_curves\x18\n \x03(\x0b\x32\'.machinerySystemStructure.EmissionCurve\x12K\n\x11\x65ngine_cycle_type\x18\x0b \x01(\x0e\x32\x30.machinerySystemStructure.Engine.EngineCycleType\x12\x16\n\x0eunit_price_usd\x18\x0c \x01(\x01\x12\x15\n\rstart_delay_s\x18\r \x01(\x01\x12\x19\n\x11turn_off_power_kw\x18\x0e \x01(\x01\x12\x0b\n\x03uid\x18\x0f \x01(\t"E\n\x14NOxCalculationMethod\x12\n\n\x06TIER_2\x10\x00\x12\n\n\x06TIER_1\x10\x01\x12\n\n\x06TIER_3\x10\x02\x12\t\n\x05\x43URVE\x10\x03"O\n\x0f\x45ngineCycleType\x12\x08\n\x04NONE\x10\x00\x12\n\n\x06\x44IESEL\x10\x01\x12\x08\n\x04OTTO\x10\x02\x12\x1c\n\x18LEAN_BURN_SPARK_IGNITION\x10\x03"\xdb\x04\n\x05\x43OGAS\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x16\n\x0erated_power_kw\x18\x02 \x01(\x01\x12\x17\n\x0frated_speed_rpm\x18\x03 \x01(\x01\x12\x38\n\nefficiency\x18\x04 \x01(\x0b\x32$.machinerySystemStructure.Efficiency\x12\x45\n\x17gas_turbine_power_curve\x18\x05 \x01(\x0b\x32$.machinerySystemStructure.PowerCurve\x12G\n\x19steam_turbine_power_curve\x18\x06 \x01(\x0b\x32$.machinerySystemStructure.PowerCurve\x12,\n\x04\x66uel\x18\x07 \x01(\x0b\x32\x1e.machinerySystemStructure.Fuel\x12+\n#order_from_switchboard_or_shaftline\x18\x08 \x01(\r\x12U\n\x16nox_calculation_method\x18\t \x01(\x0e\x32\x35.machinerySystemStructure.Engine.NOxCalculationMethod\x12@\n\x0f\x65mission_curves\x18\n \x03(\x0b\x32\'.machinerySystemStructure.EmissionCurve\x12\x16\n\x0eunit_price_usd\x18\x0b \x01(\x01\x12\x15\n\rstart_delay_s\x18\x0c \x01(\x01\x12\x19\n\x11turn_off_power_kw\x18\r \x01(\x01\x12\x0b\n\x03uid\x18\x0e \x01(\t"\xdc\x01\n\x0f\x45lectricMachine\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x16\n\x0erated_power_kw\x18\x02 \x01(\x01\x12\x17\n\x0frated_speed_rpm\x18\x03 \x01(\x01\x12\x38\n\nefficiency\x18\x04 \x01(\x0b\x32$.machinerySystemStructure.Efficiency\x12+\n#order_from_switchboard_or_shaftline\x18\x05 \x01(\r\x12\x16\n\x0eunit_price_usd\x18\x06 \x01(\x01\x12\x0b\n\x03uid\x18\x07 \x01(\t"\x8f\x03\n\x07\x42\x61ttery\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x1b\n\x13\x65nergy_capacity_kwh\x18\x02 \x01(\x01\x12\x1d\n\x15rated_charging_rate_c\x18\x03 \x01(\x01\x12 \n\x18rated_discharging_rate_c\x18\x04 \x01(\x01\x12\x1b\n\x13\x65\x66\x66iciency_charging\x18\x05 \x01(\x01\x12\x1e\n\x16\x65\x66\x66iciency_discharging\x18\x06 \x01(\x01\x12\x1f\n\x17initial_state_of_charge\x18\x07 \x01(\x01\x12+\n#order_from_switchboard_or_shaftline\x18\x08 \x01(\r\x12\x16\n\x0eunit_price_usd\x18\t \x01(\x01\x12&\n\x1eself_discharge_percent_per_day\x18\n \x01(\x01\x12\x1f\n\x17state_of_energy_minimum\x18\x0b \x01(\x01\x12\x1f\n\x17state_of_energy_maximum\x18\x0c \x01(\x01\x12\x0b\n\x03uid\x18\r \x01(\t"\xc5\x01\n\x11\x45lectricComponent\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x16\n\x0erated_power_kw\x18\x02 \x01(\x01\x12\x38\n\nefficiency\x18\x03 \x01(\x0b\x32$.machinerySystemStructure.Efficiency\x12+\n#order_from_switchboard_or_shaftline\x18\x04 \x01(\r\x12\x16\n\x0eunit_price_usd\x18\x05 \x01(\x01\x12\x0b\n\x03uid\x18\x06 \x01(\t"\xb9\x02\n\x08\x46uelCell\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x16\n\x0erated_power_kw\x18\x02 \x01(\x01\x12\x38\n\nefficiency\x18\x03 \x01(\x0b\x32$.machinerySystemStructure.Efficiency\x12+\n#order_from_switchboard_or_shaftline\x18\x05 \x01(\r\x12,\n\x04\x66uel\x18\x06 \x01(\x0b\x32\x1e.machinerySystemStructure.Fuel\x12\x16\n\x0eunit_price_usd\x18\x07 \x01(\x01\x12\x16\n\x0enumber_modules\x18\x08 \x01(\r\x12\x1e\n\x16power_minimum_specific\x18\t \x01(\x01\x12\x15\n\rstart_delay_s\x18\n \x01(\x01\x12\x0b\n\x03uid\x18\x0b \x01(\t"\xa3\x01\n\tPropeller\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x38\n\nefficiency\x18\x02 \x01(\x0b\x32$.machinerySystemStructure.Efficiency\x12\x14\n\x0cpropulsor_id\x18\x03 \x01(\r\x12+\n#order_from_switchboard_or_shaftline\x18\x05 \x01(\r\x12\x0b\n\x03uid\x18\x06 \x01(\t"$\n\nBusBreaker\x12\x16\n\x0eswitchboard_to\x18\x01 \x01(\x05"\x82\x02\n\x0eSuperCapacitor\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x1a\n\x12\x65nergy_capacity_wh\x18\x02 \x01(\x01\x12\x16\n\x0erated_power_kw\x18\x03 \x01(\x01\x12\x1b\n\x13\x65\x66\x66iciency_charging\x18\x04 \x01(\x01\x12\x1e\n\x16\x65\x66\x66iciency_discharging\x18\x05 \x01(\x01\x12\x1f\n\x17initial_state_of_charge\x18\x06 \x01(\x01\x12+\n#order_from_switchboard_or_shaftline\x18\x07 \x01(\r\x12\x16\n\x0eunit_price_usd\x18\x08 \x01(\x01\x12\x0b\n\x03uid\x18\t \x01(\t"\xc7\x01\n\x13MechanicalComponent\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x16\n\x0erated_power_kw\x18\x02 \x01(\x01\x12\x38\n\nefficiency\x18\x03 \x01(\x0b\x32$.machinerySystemStructure.Efficiency\x12+\n#order_from_switchboard_or_shaftline\x18\x04 \x01(\r\x12\x16\n\x0eunit_price_usd\x18\x05 \x01(\x01\x12\x0b\n\x03uid\x18\x06 \x01(\t"\x8d\x0e\n\tSubsystem\x12,\n\x04gear\x18\x01 \x01(\x0b\x32\x1e.machinerySystemStructure.Gear\x12\x30\n\x06\x65ngine\x18\x02 \x01(\x0b\x32 .machinerySystemStructure.Engine\x12\x43\n\x10\x65lectric_machine\x18\x03 \x01(\x0b\x32).machinerySystemStructure.ElectricMachine\x12@\n\x0btransformer\x18\x04 \x01(\x0b\x32+.machinerySystemStructure.ElectricComponent\x12?\n\nconverter1\x18\x05 \x01(\x0b\x32+.machinerySystemStructure.ElectricComponent\x12?\n\nconverter2\x18\x06 \x01(\x0b\x32+.machinerySystemStructure.ElectricComponent\x12\x32\n\x07\x62\x61ttery\x18\x07 \x01(\x0b\x32!.machinerySystemStructure.Battery\x12\x35\n\tfuel_cell\x18\x08 \x01(\x0b\x32".machinerySystemStructure.FuelCell\x12\x36\n\tpropeller\x18\t \x01(\x0b\x32#.machinerySystemStructure.Propeller\x12\x39\n\x0b\x62us_breaker\x18\n \x01(\x0b\x32$.machinerySystemStructure.BusBreaker\x12@\n\x0esupercapacitor\x18\x0b \x01(\x0b\x32(.machinerySystemStructure.SuperCapacitor\x12?\n\nother_load\x18\x0c \x01(\x0b\x32+.machinerySystemStructure.ElectricComponent\x12.\n\x05\x63ogas\x18\r \x01(\x0b\x32\x1f.machinerySystemStructure.COGAS\x12\x41\n\npower_type\x18\x0e \x01(\x0e\x32-.machinerySystemStructure.Subsystem.PowerType\x12I\n\x0e\x63omponent_type\x18\x0f \x01(\x0e\x32\x31.machinerySystemStructure.Subsystem.ComponentType\x12\x0c\n\x04name\x18\x10 \x01(\t\x12\x16\n\x0erated_power_kw\x18\x11 \x01(\x01\x12\x17\n\x0frated_speed_rpm\x18\x12 \x01(\x01\x12-\n%ramp_up_rate_limit_percent_per_second\x18\x13 \x01(\x01\x12/\n\'ramp_down_rate_limit_percent_per_second\x18\x14 \x01(\x01\x12\x17\n\x0f\x62\x61se_load_order\x18\x15 \x01(\r\x12\x0b\n\x03uid\x18\x16 \x01(\t"s\n\tPowerType\x12\t\n\x05NONE1\x10\x00\x12\x10\n\x0cPOWER_SOURCE\x10\x01\x12\x12\n\x0ePOWER_CONSUMER\x10\x02\x12\x0b\n\x07PTI_PTO\x10\x03\x12\x12\n\x0e\x45NERGY_STORAGE\x10\x04\x12\x14\n\x10SHORE_CONNECTION\x10\x05"\xbd\x04\n\rComponentType\x12\x08\n\x04NONE\x10\x00\x12\x0f\n\x0bMAIN_ENGINE\x10\x01\x12\x14\n\x10\x41UXILIARY_ENGINE\x10\x02\x12\r\n\tGENERATOR\x10\x03\x12\x14\n\x10PROPULSION_DRIVE\x10\x04\x12\x0e\n\nOTHER_LOAD\x10\x05\x12\x12\n\x0ePTI_PTO_SYSTEM\x10\x06\x12\x12\n\x0e\x42\x41TTERY_SYSTEM\x10\x07\x12\x14\n\x10\x46UEL_CELL_SYSTEM\x10\x08\x12\r\n\tRECTIFIER\x10\t\x12\x1c\n\x18MAIN_ENGINE_WITH_GEARBOX\x10\n\x12\x12\n\x0e\x45LECTRIC_MOTOR\x10\x0b\x12\n\n\x06GENSET\x10\x0c\x12\x0f\n\x0bTRANSFORMER\x10\r\x12\x0c\n\x08INVERTER\x10\x0e\x12\x13\n\x0f\x43IRCUIT_BREAKER\x10\x0f\x12\x14\n\x10\x41\x43TIVE_FRONT_END\x10\x10\x12\x13\n\x0fPOWER_CONVERTER\x10\x11\x12\x17\n\x13SYNCHRONOUS_MACHINE\x10\x12\x12\x15\n\x11INDUCTION_MACHINE\x10\x13\x12\x0b\n\x07GEARBOX\x10\x14\x12\r\n\tFUEL_CELL\x10\x15\x12\x12\n\x0ePROPELLER_LOAD\x10\x16\x12\x19\n\x15OTHER_MECHANICAL_LOAD\x10\x17\x12\x0b\n\x07\x42\x41TTERY\x10\x18\x12\x12\n\x0eSUPERCAPACITOR\x10\x19\x12\x19\n\x15SUPERCAPACITOR_SYSTEM\x10\x1a\x12\x0f\n\x0bSHORE_POWER\x10\x1b\x12\t\n\x05\x43OGAS\x10\x1c\x12\t\n\x05\x43OGES\x10\x1d"^\n\x0bSwitchboard\x12\x16\n\x0eswitchboard_id\x18\x01 \x01(\r\x12\x37\n\nsubsystems\x18\x02 \x03(\x0b\x32#.machinerySystemStructure.Subsystem"[\n\tShaftLine\x12\x15\n\rshaft_line_id\x18\x01 \x01(\r\x12\x37\n\nsubsystems\x18\x02 \x03(\x0b\x32#.machinerySystemStructure.Subsystem"L\n\x10MechanicalSystem\x12\x38\n\x0bshaft_lines\x18\x01 \x03(\x0b\x32#.machinerySystemStructure.ShaftLine"M\n\x0e\x45lectricSystem\x12;\n\x0cswitchboards\x18\x01 \x03(\x0b\x32%.machinerySystemStructure.Switchboard"Y\n\x0b\x46uelStorage\x12\x35\n\tfuel_type\x18\x01 \x01(\x0e\x32".machinerySystemStructure.FuelType\x12\x13\n\x0b\x63\x61pacity_kg\x18\x02 \x01(\x01"\xfe\x03\n\x0fMachinerySystem\x12\x0c\n\x04name\x18\x01 \x01(\t\x12Q\n\x0fpropulsion_type\x18\x02 \x01(\x0e\x32\x38.machinerySystemStructure.MachinerySystem.PropulsionType\x12;\n\x0c\x66uel_storage\x18\x03 \x03(\x0b\x32%.machinerySystemStructure.FuelStorage\x12.\n&maximum_allowed_genset_load_percentage\x18\x04 \x01(\x01\x12\x45\n\x11mechanical_system\x18\x05 \x01(\x0b\x32*.machinerySystemStructure.MechanicalSystem\x12\x41\n\x0f\x65lectric_system\x18\x06 \x01(\x0b\x32(.machinerySystemStructure.ElectricSystem\x12\x31\n)maximum_allowed_fuel_cell_load_percentage\x18\x07 \x01(\x01\x12$\n\x1c\x61verage_base_load_percentage\x18\x08 \x01(\x01":\n\x0ePropulsionType\x12\x0e\n\nMECHANICAL\x10\x00\x12\x0c\n\x08\x45LECTRIC\x10\x01\x12\n\n\x06HYBRID\x10\x02*T\n\x0c\x45missionType\x12\x08\n\x04NONE\x10\x00\x12\x07\n\x03SOX\x10\x01\x12\x07\n\x03NOX\x10\x02\x12\x06\n\x02\x43O\x10\x03\x12\x06\n\x02PM\x10\x04\x12\x06\n\x02HC\x10\x05\x12\x07\n\x03\x43H4\x10\x06\x12\x07\n\x03N2O\x10\x07*\xc6\x01\n\x08\x46uelType\x12\n\n\x06\x44IESEL\x10\x00\x12\x07\n\x03HFO\x10\x01\x12\x0f\n\x0bNATURAL_GAS\x10\x02\x12\x0c\n\x08HYDROGEN\x10\x03\x12\x0b\n\x07\x41MMONIA\x10\x04\x12\x0f\n\x0bLPG_PROPANE\x10\x05\x12\x0e\n\nLPG_BUTANE\x10\x06\x12\x0b\n\x07\x45THANOL\x10\x07\x12\x0c\n\x08METHANOL\x10\x08\x12\x07\n\x03LFO\x10\t\x12\x0e\n\nLSFO_CRUDE\x10\n\x12\x0e\n\nLSFO_BLEND\x10\x0b\x12\t\n\x05ULSFO\x10\x0c\x12\t\n\x05VLSFO\x10\r*C\n\nFuelOrigin\x12\t\n\x05NONE1\x10\x00\x12\n\n\x06\x46OSSIL\x10\x01\x12\x07\n\x03\x42IO\x10\x02\x12\x15\n\x11RENEWABLE_NON_BIO\x10\x03*E\n\x0f\x46uelSpecifiedBy\x12\t\n\x05NONE2\x10\x00\x12\x14\n\x10\x46UEL_EU_MARITIME\x10\x01\x12\x07\n\x03IMO\x10\x02\x12\x08\n\x04USER\x10\x03\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "system_structure_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_EMISSIONTYPE"]._serialized_start = 7458
    _globals["_EMISSIONTYPE"]._serialized_end = 7542
    _globals["_FUELTYPE"]._serialized_start = 7545
    _globals["_FUELTYPE"]._serialized_end = 7743
    _globals["_FUELORIGIN"]._serialized_start = 7745
    _globals["_FUELORIGIN"]._serialized_end = 7812
    _globals["_FUELSPECIFIEDBY"]._serialized_start = 7814
    _globals["_FUELSPECIFIEDBY"]._serialized_end = 7883
    _globals["_POINT"]._serialized_start = 52
    _globals["_POINT"]._serialized_end = 81
    _globals["_CURVE1D"]._serialized_start = 83
    _globals["_CURVE1D"]._serialized_end = 141
    _globals["_BSFCCURVE"]._serialized_start = 143
    _globals["_BSFCCURVE"]._serialized_end = 238
    _globals["_EFFICIENCYCURVE"]._serialized_start = 240
    _globals["_EFFICIENCYCURVE"]._serialized_end = 341
    _globals["_BSFC"]._serialized_start = 343
    _globals["_BSFC"]._serialized_end = 431
    _globals["_EFFICIENCY"]._serialized_start = 433
    _globals["_EFFICIENCY"]._serialized_end = 548
    _globals["_POWERCURVE"]._serialized_start = 550
    _globals["_POWERCURVE"]._serialized_end = 646
    _globals["_PROPULSIONPOWERTIMESERIES"]._serialized_start = 649
    _globals["_PROPULSIONPOWERTIMESERIES"]._serialized_end = 782
    _globals["_AUXILIARYLOADTIMESERIES"]._serialized_start = 785
    _globals["_AUXILIARYLOADTIMESERIES"]._serialized_end = 918
    _globals["_AUXILIARYLOAD"]._serialized_start = 920
    _globals["_AUXILIARYLOAD"]._serialized_end = 976
    _globals["_EMISSIONCURVE"]._serialized_start = 979
    _globals["_EMISSIONCURVE"]._serialized_end = 1141
    _globals["_GEAR"]._serialized_start = 1144
    _globals["_GEAR"]._serialized_end = 1373
    _globals["_FUEL"]._serialized_start = 1375
    _globals["_FUEL"]._serialized_end = 1495
    _globals["_ENGINE"]._serialized_start = 1498
    _globals["_ENGINE"]._serialized_end = 2284
    _globals["_ENGINE_NOXCALCULATIONMETHOD"]._serialized_start = 2134
    _globals["_ENGINE_NOXCALCULATIONMETHOD"]._serialized_end = 2203
    _globals["_ENGINE_ENGINECYCLETYPE"]._serialized_start = 2205
    _globals["_ENGINE_ENGINECYCLETYPE"]._serialized_end = 2284
    _globals["_COGAS"]._serialized_start = 2287
    _globals["_COGAS"]._serialized_end = 2890
    _globals["_ELECTRICMACHINE"]._serialized_start = 2893
    _globals["_ELECTRICMACHINE"]._serialized_end = 3113
    _globals["_BATTERY"]._serialized_start = 3116
    _globals["_BATTERY"]._serialized_end = 3515
    _globals["_ELECTRICCOMPONENT"]._serialized_start = 3518
    _globals["_ELECTRICCOMPONENT"]._serialized_end = 3715
    _globals["_FUELCELL"]._serialized_start = 3718
    _globals["_FUELCELL"]._serialized_end = 4031
    _globals["_PROPELLER"]._serialized_start = 4034
    _globals["_PROPELLER"]._serialized_end = 4197
    _globals["_BUSBREAKER"]._serialized_start = 4199
    _globals["_BUSBREAKER"]._serialized_end = 4235
    _globals["_SUPERCAPACITOR"]._serialized_start = 4238
    _globals["_SUPERCAPACITOR"]._serialized_end = 4496
    _globals["_MECHANICALCOMPONENT"]._serialized_start = 4499
    _globals["_MECHANICALCOMPONENT"]._serialized_end = 4698
    _globals["_SUBSYSTEM"]._serialized_start = 4701
    _globals["_SUBSYSTEM"]._serialized_end = 6506
    _globals["_SUBSYSTEM_POWERTYPE"]._serialized_start = 5815
    _globals["_SUBSYSTEM_POWERTYPE"]._serialized_end = 5930
    _globals["_SUBSYSTEM_COMPONENTTYPE"]._serialized_start = 5933
    _globals["_SUBSYSTEM_COMPONENTTYPE"]._serialized_end = 6506
    _globals["_SWITCHBOARD"]._serialized_start = 6508
    _globals["_SWITCHBOARD"]._serialized_end = 6602
    _globals["_SHAFTLINE"]._serialized_start = 6604
    _globals["_SHAFTLINE"]._serialized_end = 6695
    _globals["_MECHANICALSYSTEM"]._serialized_start = 6697
    _globals["_MECHANICALSYSTEM"]._serialized_end = 6773
    _globals["_ELECTRICSYSTEM"]._serialized_start = 6775
    _globals["_ELECTRICSYSTEM"]._serialized_end = 6852
    _globals["_FUELSTORAGE"]._serialized_start = 6854
    _globals["_FUELSTORAGE"]._serialized_end = 6943
    _globals["_MACHINERYSYSTEM"]._serialized_start = 6946
    _globals["_MACHINERYSYSTEM"]._serialized_end = 7456
    _globals["_MACHINERYSYSTEM_PROPULSIONTYPE"]._serialized_start = 7398
    _globals["_MACHINERYSYSTEM_PROPULSIONTYPE"]._serialized_end = 7456
# @@protoc_insertion_point(module_scope)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../01_ConvertToProtobuf.ipynb.

# %% auto 0
__all__ = [
    "convert_efficiency_curve_to_protobuf",
    "convert_np_array_to_protobuf_power_curve",
    "convert_bsfc_curve_to_protobuf",
    "convert_electric_machine_to_protobuf",
    "convert_electric_component_to_protobuf",
    "convert_battery_component_to_protobuf",
    "convert_supercapacitor_component_to_protobuf",
    "convert_serial_electric_system_to_protobuf",
    "convert_nox_calculation_method_to_protobuf",
    "convert_emission_curves_to_protobuf",
    "convert_engine_component_to_protobuf",
    "convert_cogas_component_to_protobuf",
    "convert_switchboard_to_protobuf",
    "convert_shaftline_to_protobuf",
    "convert_electric_system_to_protobuf",
    "convert_electric_system_to_protobuf_machinery_system",
    "convert_mechanical_system_to_protobuf",
    "convert_mechanical_propulsion_system_with_electric_system_to_protobuf",
    "convert_hybrid_propulsion_system_to_protobuf",
]

# %% ../01_ConvertToProtobuf.ipynb 3
from feems.components_model import (
    Engine,
    Switchboard,
    ShaftLine,
    MainEngineForMechanicalPropulsion,
    MechanicalPropulsionComponent,
    MainEngineWithGearBoxForMechanicalPropulsion,
)
from typing import cast, Union, List

from feems.components_model.component_base import BasicComponent
from feems.types_for_feems import TypeComponent, NOxCalculationMethod, EmissionCurve
from feems.components_model.component_mechanical import COGAS, EngineDualFuel
from feems.components_model.component_electric import (
    COGES,
    ElectricComponent,
    FuelCellSystem,
    ElectricMachine,
    BatterySystem,
    Battery,
    SuperCapacitorSystem,
    PTIPTO,
    SuperCapacitor,
)
from feems.components_model.component_electric import SerialSystemElectric, Genset
from feems.system_model import (
    ElectricPowerSystem,
    MechanicalPropulsionSystem,
    MechanicalPropulsionSystemWithElectricPowerSystem,
    HybridPropulsionSystem,
)
import numpy as np

import MachSysS.system_structure_pb2 as proto


def convert_efficiency_curve_to_protobuf(
    component: ElectricComponent,
) -> proto.Efficiency:
    """Convert efficiency value or curve in the component to protobuf message"""
    efficiency = proto.Efficiency()
    if len(component._efficiency_points) == 1:
        efficiency.value = component._efficiency_points[0]
    else:
        efficiency.curve.curve.points.extend(
            [
                proto.Point(x=each_point[0], y=each_point[1])
                for each_point in component._efficiency_points
            ]
        )
        efficiency.curve.x_label = "power load"
        efficiency.curve.y_label = "efficiency"
    return efficiency


def convert_np_array_to_protobuf_power_curve(power_curve: np.array) -> proto.PowerCurve:
    """Convert power curve in the component to protobuf message"""
    # Check if the array is in n x 2 dimension or a single value
    if power_curve.shape[1] == 2:
        curve = proto.Curve1D()
        curve.points.extend(
            [
                proto.Point(x=each_point[0], y=each_point[1])
                for each_point in power_curve
            ]
        )
        return proto.PowerCurve(
            x_label="load_ratio",
            y_label="power_kw",
            curve=curve,
        )
    else:
        raise ValueError(
            f"The power curve array should have 2 columns. The array has {power_curve.shape[1]} columns."
        )


def convert_bsfc_curve_to_protobuf(
    component: Union[Engine, EngineDualFuel], for_pilot_fuel: bool = False
) -> proto.BSFC:
    """Convert bsfc value or curve in the component to protobuf message"""
    bsfc = proto.BSFC()
    bsfc_points = (
        component.specific_fuel_consumption_points
        if not for_pilot_fuel
        else component.specific_pilot_fuel_consumption_points
    )
    if len(bsfc_points) == 1:
        bsfc.value = bsfc_points[0]
    else:
        bsfc.curve.curve.points.extend(
            [
                proto.Point(x=each_point[0], y=each_point[1])
                for each_point in bsfc_points
            ]
        )
        bsfc.curve.x_label = "power load"
        bsfc.curve.y_label = "bsfc"
    return bsfc


def convert_electric_machine_to_protobuf(
    component: ElectricMachine, order_from_switchboard: int = 1
) -> proto.ElectricMachine:
    """Convert elecrtic machine component of FEEMS to protobuf message"""
    return proto.ElectricMachine(
        name=component.name,
        rated_power_kw=component.rated_power,
        rated_speed_rpm=component.rated_speed,
        efficiency=convert_efficiency_curve_to_protobuf(component),
        order_from_switchboard_or_shaftline=order_from_switchboard,
        uid=component.uid,
    )


def convert_electric_component_to_protobuf(
    component: ElectricComponent, order_from_switchboard: int = 1
) -> proto.ElectricMachine:
    """Convert converter component of FEEMS to protobuf message"""
    return proto.ElectricComponent(
        name=component.name,
        rated_power_kw=component.rated_power,
        efficiency=convert_efficiency_curve_to_protobuf(component),
        order_from_switchboard_or_shaftline=order_from_switchboard,
        uid=component.uid,
    )


def convert_battery_component_to_protobuf(
    component: Battery, order_from_switchboard: int = 1
) -> proto.ElectricMachine:
    """Convert battery component of FEEMS to protobuf message"""
    return proto.Battery(
        name=component.name,
        energy_capacity_kwh=component.rated_capacity_kWh,
        rated_charging_rate_c=component.charging_rate_C,
        rated_discharging_rate_c=component.discharging_rate_C,
        efficiency_charging=component.eff_charging,
        efficiency_discharging=component.eff_discharging,
        initial_state_of_charge=component.soc0,
        order_from_switchboard_or_shaftline=order_from_switchboard,
        uid=component.uid,
    )


def convert_supercapacitor_component_to_protobuf(
    component: SuperCapacitor, order_from_switchboard: int = 1
) -> proto.ElectricMachine:
    """Convert converter component of FEEMS to protobuf message"""
    return proto.SuperCapacitor(
        name=component.name,
        energy_capacity_wh=component.rated_capacity_Wh,
        rated_power_kw=component.rated_power,
        efficiency_charging=component.eff_charging,
        efficiency_discharging=component.eff_discharging,
        initial_state_of_charge=component.soc0,
        order_from_switchboard_or_shaftline=order_from_switchboard,
        uid=component.uid,
    )


def convert_serial_electric_system_to_protobuf(
    component: Union[SerialSystemElectric, PTIPTO],
    initial_order_from_switchboard: int = 1,
) -> proto.Subsystem:
    """Convert serial electric system or PTI/PTO component to protobuf message"""
    order = initial_order_from_switchboard
    subsystem = proto.Subsystem(
        name=component.name,
        rated_power_kw=component.rated_power,
        rated_speed_rpm=component.rated_speed,
        component_type=component.type.value,
        power_type=component.power_type.value,
        uid=component.uid,
    )
    for subcomponent in component.components:
        if subcomponent.type == TypeComponent.TRANSFORMER:
            subsystem.transformer.CopyFrom(
                convert_electric_component_to_protobuf(
                    component=subcomponent, order_from_switchboard=order
                )
            )
        if subcomponent.type in [
            TypeComponent.POWER_CONVERTER,
            TypeComponent.INVERTER,
            TypeComponent.RECTIFIER,
            TypeComponent.ACTIVE_FRONT_END,
        ]:
            if not subsystem.HasField("converter1"):
                subsystem.converter1.CopyFrom(
                    convert_electric_component_to_protobuf(
                        component=subcomponent, order_from_switchboard=order
                    )
                )
            else:
                subsystem.converter2.CopyFrom(
                    convert_electric_component_to_protobuf(
                        component=subcomponent, order_from_switchboard=order
                    )
                )
        if subcomponent.type in [
            TypeComponent.SYNCHRONOUS_MACHINE,
            TypeComponent.INDUCTION_MACHINE,
            TypeComponent.ELECTRIC_MOTOR,
        ]:
            subsystem.electric_machine.CopyFrom(
                convert_electric_machine_to_protobuf(
                    component=subcomponent, order_from_switchboard=order
                )
            )
        order += 1
    return subsystem


def convert_nox_calculation_method_to_protobuf(
    nox_calculation_method_feems: NOxCalculationMethod,
) -> proto.Engine.NOxCalculationMethod:
    """Convert nox calculation method of FEEMS to protobuf message"""
    index = proto.Engine.NOxCalculationMethod.Value(nox_calculation_method_feems.name)
    return index


def convert_emission_curves_to_protobuf(
    emission_curves_feems: List[EmissionCurve],
) -> List[proto.EmissionCurve]:
    """Convert emission curves of FEEMS to protobuf message"""
    if emission_curves_feems is None:
        return []
    return [
        proto.EmissionCurve(
            x_label="load_ratio",
            y_label="emission_g_per_kwh",
            curve=proto.Curve1D(
                points=[
                    proto.Point(
                        x=each_point.load_ratio, y=each_point.emission_g_per_kwh
                    )
                    for each_point in each_curve.points_per_kwh
                ]
            ),
            emission_type=each_curve.emission.value,
        )
        for each_curve in emission_curves_feems
    ]


def convert_engine_component_to_protobuf(
    engine_feems: Union[Engine, EngineDualFuel],
    order_from_shaftline_or_switchboard: int = 1,
) -> proto.Engine:
    """Convert engine component of FEEMS to protobuf message"""
    engine = proto.Engine(
        name=engine_feems.name,
        rated_power_kw=engine_feems.rated_power,
        rated_speed_rpm=engine_feems.rated_speed,
        bsfc=convert_bsfc_curve_to_protobuf(engine_feems),
        main_fuel=proto.Fuel(
            fuel_type=engine_feems.fuel_type.value,
            fuel_origin=engine_feems.fuel_origin.value,
        ),
        nox_calculation_method=convert_nox_calculation_method_to_protobuf(
            engine_feems.nox_calculation_method
        ),
        emission_curves=convert_emission_curves_to_protobuf(
            engine_feems.emission_curves
        ),
        engine_cycle_type=engine_feems.engine_cycle_type.value,
        order_from_switchboard_or_shaftline=order_from_shaftline_or_switchboard,
        uid=engine_feems.uid,
    )
    if isinstance(engine_feems, EngineDualFuel):
        engine.pilot_bsfc.CopyFrom(
            convert_bsfc_curve_to_protobuf(engine_feems, for_pilot_fuel=True)
        )
        engine.pilot_fuel.CopyFrom(
            proto.Fuel(
                fuel_type=engine_feems.pilot_fuel_type.value,
                fuel_origin=engine_feems.pilot_fuel_origin.value,
            )
        )
    return engine


def convert_cogas_component_to_protobuf(
    component: COGAS,
    order_from_shaftline_or_switchboard: int = 1,
) -> proto.Engine:
    """Convert engine component of FEEMS to protobuf message"""
    cogas = proto.COGAS(
        name=component.name,
        rated_power_kw=component.rated_power,
        rated_speed_rpm=component.rated_speed,
        efficiency=convert_efficiency_curve_to_protobuf(component),
        fuel=proto.Fuel(
            fuel_type=component.fuel_type.value,
            fuel_origin=component.fuel_origin.value,
        ),
        nox_calculation_method=convert_nox_calculation_method_to_protobuf(
            component.nox_calculation_method
        ),
        emission_curves=convert_emission_curves_to_protobuf(component.emission_curves),
        order_from_switchboard_or_shaftline=order_from_shaftline_or_switchboard,
        uid=component.uid,
    )
    if component.gas_turbine_power_curve is not None:
        cogas.gas_turbine_power_curve.CopyFrom(
            convert_np_array_to_protobuf_power_curve(component.gas_turbine_power_curve)
        )
        cogas.steam_turbine_power_curve.CopyFrom(
            convert_np_array_to_protobuf_power_curve(
                component.steam_turbine_power_curve
            )
        )
    return cogas


def convert_switchboard_to_protobuf(
    switchboard_feems: Switchboard,
) -> proto.Switchboard:
    switchboard_proto = proto.Switchboard()
    switchboard_proto.switchboard_id = switchboard_feems.id

    for component in switchboard_feems.components:
        subsystem = proto.Subsystem(
            power_type=component.power_type.value,
            component_type=component.type.value,
            name=component.name,
            rated_power_kw=component.rated_power,
            rated_speed_rpm=component.rated_speed,
            uid=component.uid,
        )
        if component.type == TypeComponent.GENERATOR:
            subsystem.electric_machine.CopyFrom(
                convert_electric_machine_to_protobuf(
                    component=component,
                )
            )
        elif component.type == TypeComponent.FUEL_CELL_SYSTEM:
            component = cast(FuelCellSystem, component)
            subsystem.converter1.CopyFrom(
                convert_electric_component_to_protobuf(component=component.converter)
            )
            subsystem.fuel_cell.CopyFrom(
                proto.FuelCell(
                    name=component.fuel_cell.name,
                    rated_power_kw=component.fuel_cell.rated_power,
                    efficiency=convert_efficiency_curve_to_protobuf(
                        component.fuel_cell
                    ),
                    fuel=proto.Fuel(
                        fuel_type=component.fuel_cell.fuel_type.value,
                        fuel_origin=component.fuel_cell.fuel_origin.value,
                    ),
                    number_modules=component.number_modules,
                    order_from_switchboard_or_shaftline=2,
                    uid=component.fuel_cell.uid,
                )
            )
        elif component.type == TypeComponent.COGES:
            component = cast(COGES, component)
            subsystem.cogas.CopyFrom(
                convert_cogas_component_to_protobuf(
                    component=component.cogas, order_from_shaftline_or_switchboard=2
                )
            )
            subsystem.electric_machine.CopyFrom(
                convert_electric_machine_to_protobuf(
                    component=component.generator, order_from_switchboard=1
                )
            )
        elif component.type == TypeComponent.GENSET:
            component = cast(Genset, component)
            subsystem.electric_machine.CopyFrom(
                convert_electric_machine_to_protobuf(
                    component=component.generator,
                )
            )
            subsystem.engine.CopyFrom(
                convert_engine_component_to_protobuf(
                    engine_feems=component.aux_engine,
                    order_from_shaftline_or_switchboard=2,
                )
            )
        elif component.type == TypeComponent.OTHER_LOAD:
            subsystem.other_load.CopyFrom(
                convert_electric_component_to_protobuf(component=component)
            )
        elif component.type in [
            TypeComponent.PTI_PTO_SYSTEM,
            TypeComponent.PROPULSION_DRIVE,
        ]:
            subsystem.MergeFrom(
                convert_serial_electric_system_to_protobuf(component=component)
            )
        elif component.type == TypeComponent.BATTERY_SYSTEM:
            component = cast(BatterySystem, component)
            subsystem.converter1.CopyFrom(
                convert_electric_component_to_protobuf(component=component.converter)
            )
            subsystem.battery.CopyFrom(
                convert_battery_component_to_protobuf(
                    component=component.battery, order_from_switchboard=2
                )
            )
        elif component.type == TypeComponent.BATTERY:
            subsystem.battery.CopyFrom(
                convert_battery_component_to_protobuf(component=component)
            )
        elif component.type == TypeComponent.SUPERCAPACITOR_SYSTEM:
            component = cast(SuperCapacitorSystem, component)
            subsystem.converter1.CopyFrom(
                convert_electric_component_to_protobuf(component=component.converter)
            )
            subsystem.battery.CopyFrom(
                convert_supercapacitor_component_to_protobuf(
                    component=component.supercapacitor, order_from_switchboard=2
                )
            )
        elif component.type == TypeComponent.SUPERCAPACITOR:
            subsystem.battery.CopyFrom(
                convert_supercapacitor_component_to_protobuf(component=component)
            )
        else:
            raise TypeError(
                f"The component type ({component.type.name}) is not a proper type for an electric "
                f"system or the conversion for the type is not implemented."
            )
        switchboard_proto.subsystems.append(subsystem)
    return switchboard_proto


def convert_shaftline_to_protobuf(shaftline_feems: ShaftLine) -> proto.ShaftLine:
    """Convert shaft line to protobuf message"""
    shaftline_proto = proto.ShaftLine()
    shaftline_proto.shaft_line_id = shaftline_feems.id
    gear_proto = None
    propeller_id = 1
    # If there is a gear box component on the shaft line, it is recognized as a gear box connecting
    # the main engine to the propeller and pti/pto.
    # The gear box is added to the protobuf message first.
    for gear in filter(
        lambda component: component.type == TypeComponent.GEARBOX,
        shaftline_feems.components,
    ):
        gear = cast(MechanicalPropulsionComponent, gear)
        gear_proto = proto.Gear(
            name=gear.name,
            rated_power_kw=gear.rated_power,
            rated_speed_rpm=gear.rated_speed,
            efficiency=convert_efficiency_curve_to_protobuf(gear),
            order_from_switchboard_or_shaftline=1,
            uid=gear.uid,
        )

    for component in shaftline_feems.components:
        subsystem = proto.Subsystem(
            power_type=component.power_type.value,
            component_type=component.type.value,
            name=component.name,
            rated_power_kw=component.rated_power,
            rated_speed_rpm=component.rated_speed,
            uid=component.uid,
        )
        if component.type == TypeComponent.MAIN_ENGINE:
            component = cast(MainEngineForMechanicalPropulsion, component)
            subsystem.engine.CopyFrom(
                convert_engine_component_to_protobuf(
                    engine_feems=component.engine, order_from_shaftline_or_switchboard=1
                )
            )
        elif component.type == TypeComponent.MAIN_ENGINE_WITH_GEARBOX:
            if gear_proto is not None:
                raise ValueError(
                    f"The shaft line {shaftline_feems.id} already has a gear box. "
                    f"Please use a main-engine component rather than a main-engine-with-a-gear-box"
                    f"component."
                )
            component = cast(MainEngineWithGearBoxForMechanicalPropulsion, component)
            subsystem.engine.CopyFrom(
                convert_engine_component_to_protobuf(
                    engine_feems=component.main_engine,
                    order_from_shaftline_or_switchboard=2,
                )
            )
            subsystem.gear.CopyFrom(
                proto.Gear(
                    name=component.gearbox.name,
                    rated_power_kw=component.gearbox.rated_power,
                    rated_speed_rpm=component.gearbox.rated_speed,
                    efficiency=convert_efficiency_curve_to_protobuf(component.gearbox),
                    order_from_switchboard_or_shaftline=1,
                    uid=component.gearbox.uid,
                )
            )
        elif component.type == TypeComponent.PROPELLER_LOAD:
            if gear_proto is not None:
                subsystem.gear.CopyFrom(gear_proto)
            subsystem.propeller.CopyFrom(
                proto.Propeller(
                    efficiency=convert_efficiency_curve_to_protobuf(component),
                    propulsor_id=propeller_id,
                    order_from_switchboard_or_shaftline=2,
                    uid=component.uid,
                )
            )
            propeller_id += 1
        elif component.type == TypeComponent.PTI_PTO_SYSTEM:
            subsystem.MergeFrom(
                convert_serial_electric_system_to_protobuf(component=component)
            )
        elif component.type == TypeComponent.GEARBOX:
            continue
        else:
            raise ValueError(
                f"The shaftline contains a component ({component.name}) that has an "
                f"imcompatible type ({component.type}) for conversion."
            )
        shaftline_proto.subsystems.append(subsystem)
    return shaftline_proto


# %% ../01_ConvertToProtobuf.ipynb 6
def convert_electric_system_to_protobuf(
    electric_system: ElectricPowerSystem,
) -> proto.ElectricSystem:
    """Convert electric system to protobuf message"""
    return proto.ElectricSystem(
        switchboards=[
            convert_switchboard_to_protobuf(switchboard)
            for _, switchboard in electric_system.switchboards.items()
        ]
    )


def convert_electric_system_to_protobuf_machinery_system(
    electric_system: ElectricPowerSystem,
    maximum_allowed_genset_load_percentage: float = 80.0,
) -> proto.MachinerySystem:
    """Convert electric system to protobuf message as a machinery system"""
    return proto.MachinerySystem(
        name=electric_system.name,
        propulsion_type=proto.MachinerySystem.PropulsionType.ELECTRIC,
        fuel_storage=[],
        maximum_allowed_genset_load_percentage=maximum_allowed_genset_load_percentage,
        electric_system=convert_electric_system_to_protobuf(electric_system),
    )


def convert_mechanical_system_to_protobuf(
    mechanical_propulsion_system: MechanicalPropulsionSystem,
) -> proto.MechanicalSystem:
    return proto.MechanicalSystem(
        shaft_lines=[
            convert_shaftline_to_protobuf(shaftline)
            for shaftline in mechanical_propulsion_system.shaft_line
        ]
    )


def convert_mechanical_propulsion_system_with_electric_system_to_protobuf(
    system_feems: MechanicalPropulsionSystemWithElectricPowerSystem,
    maximum_allowed_genset_load_percentage: float = 80.0,
) -> proto.MachinerySystem:
    return proto.MachinerySystem(
        name=system_feems.name,
        propulsion_type=proto.MachinerySystem.PropulsionType.MECHANICAL,
        fuel_storage=[],
        maximum_allowed_genset_load_percentage=maximum_allowed_genset_load_percentage,
        mechanical_system=convert_mechanical_system_to_protobuf(
            system_feems.mechanical_system
        ),
        electric_system=convert_electric_system_to_protobuf(
            system_feems.electric_system
        ),
    )


def convert_hybrid_propulsion_system_to_protobuf(
    system_feems: HybridPropulsionSystem,
    maximum_allowed_genset_load_percentage: float = 80.0,
) -> proto.MachinerySystem:
    return proto.MachinerySystem(
        name=system_feems.name,
        propulsion_type=proto.MachinerySystem.HYBRID,
        fuel_storage=[],
        maximum_allowed_genset_load_percentage=maximum_allowed_genset_load_percentage,
        mechanical_system=convert_mechanical_system_to_protobuf(
            system_feems.mechanical_system
        ),
        electric_system=convert_electric_system_to_protobuf(
            system_feems.electric_system
        ),
    )



from . import system_structure_pb2 as _system_structure_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import (
    ClassVar as _ClassVar,
    Iterable as _Iterable,
    Mapping as _Mapping,
    Optional as _Optional,
    Union as _Union,
)

DESCRIPTOR: _descriptor.FileDescriptor

class FuelScalar(_message.Message):
    __slots__ = [
        "fuel_type",
        "fuel_origin",
        "fuel_specified_by",
        "mass_or_mass_fraction",
        "lhv_mj_per_g",
        "ghg_emission_factor_well_to_tank_gco2eq_per_mj",
        "ghg_emission_factor_tank_to_wake",
    ]
    FUEL_TYPE_FIELD_NUMBER: _ClassVar[int]
    FUEL_ORIGIN_FIELD_NUMBER: _ClassVar[int]
    FUEL_SPECIFIED_BY_FIELD_NUMBER: _ClassVar[int]
    MASS_OR_MASS_FRACTION_FIELD_NUMBER: _ClassVar[int]
    LHV_MJ_PER_G_FIELD_NUMBER: _ClassVar[int]
    GHG_EMISSION_FACTOR_WELL_TO_TANK_GCO2EQ_PER_MJ_FIELD_NUMBER: _ClassVar[int]
    GHG_EMISSION_FACTOR_TANK_TO_WAKE_FIELD_NUMBER: _ClassVar[int]
    fuel_type: _system_structure_pb2.FuelType
    fuel_origin: _system_structure_pb2.FuelOrigin
    fuel_specified_by: _system_structure_pb2.FuelSpecifiedBy
    mass_or_mass_fraction: float
    lhv_mj_per_g: float
    ghg_emission_factor_well_to_tank_gco2eq_per_mj: float
    ghg_emission_factor_tank_to_wake: float
    def __init__(
        self,
        fuel_type: _Optional[_Union[_system_structure_pb2.FuelType, str]] = ...,
        fuel_origin: _Optional[_Union[_system_structure_pb2.FuelOrigin, str]] = ...,
        fuel_specified_by: _Optional[
            _Union[_system_structure_pb2.FuelSpecifiedBy, str]
        ] = ...,
        mass_or_mass_fraction: _Optional[float] = ...,
        lhv_mj_per_g: _Optional[float] = ...,
        ghg_emission_factor_well_to_tank_gco2eq_per_mj: _Optional[float] = ...,
        ghg_emission_factor_tank_to_wake: _Optional[float] = ...,
    ) -> None: ...

class FuelArray(_message.Message):
    __slots__ = [
        "fuel_type",
        "fuel_origin",
        "fuel_specified_by",
        "mass_or_mass_fraction",
        "lhv_mj_per_g",
        "ghg_emission_factor_well_to_tank_gco2eq_per_mj",
        "ghg_emission_factor_tank_to_wake",
    ]
    FUEL_TYPE_FIELD_NUMBER: _ClassVar[int]
    FUEL_ORIGIN_FIELD_NUMBER: _ClassVar[int]
    FUEL_SPECIFIED_BY_FIELD_NUMBER: _ClassVar[int]
    MASS_OR_MASS_FRACTION_FIELD_NUMBER: _ClassVar[int]
    LHV_MJ_PER_G_FIELD_NUMBER: _ClassVar[int]
    GHG_EMISSION_FACTOR_WELL_TO_TANK_GCO2EQ_PER_MJ_FIELD_NUMBER: _ClassVar[int]
    GHG_EMISSION_FACTOR_TANK_TO_WAKE_FIELD_NUMBER: _ClassVar[int]
    fuel_type: _system_structure_pb2.FuelType
    fuel_origin: _system_structure_pb2.FuelOrigin
    fuel_specified_by: _system_structure_pb2.FuelSpecifiedBy
    mass_or_mass_fraction: _containers.RepeatedScalarFieldContainer[float]
    lhv_mj_per_g: float
    ghg_emission_factor_well_to_tank_gco2eq_per_mj: float
    ghg_emission_factor_tank_to_wake: float
    def __init__(
        self,
        fuel_type: _Optional[_Union[_system_structure_pb2.FuelType, str]] = ...,
        fuel_origin: _Optional[_Union[_system_structure_pb2.FuelOrigin, str]] = ...,
        fuel_specified_by: _Optional[
            _Union[_system_structure_pb2.FuelSpecifiedBy, str]
        ] = ...,
        mass_or_mass_fraction: _Optional[_Iterable[float]] = ...,
        lhv_mj_per_g: _Optional[float] = ...,
        ghg_emission_factor_well_to_tank_gco2eq_per_mj: _Optional[float] = ...,
        ghg_emission_factor_tank_to_wake: _Optional[float] = ...,
    ) -> None: ...

class FuelConsumptionScalar(_message.Message):
    __slots__ = ["fuels"]
    FUELS_FIELD_NUMBER: _ClassVar[int]
    fuels: _containers.RepeatedCompositeFieldContainer[FuelScalar]
    def __init__(
        self, fuels: _Optional[_Iterable[_Union[FuelScalar, _Mapping]]] = ...
    ) -> None: ...

class FuelConsumptionRateArray(_message.Message):
    __slots__ = ["fuels"]
    FUELS_FIELD_NUMBER: _ClassVar[int]
    fuels: _containers.RepeatedCompositeFieldContainer[FuelArray]
    def __init__(
        self, fuels: _Optional[_Iterable[_Union[FuelArray, _Mapping]]] = ...
    ) -> None: ...

class TimeSeriesResultForComponent(_message.Message):
    __slots__ = ["time", "power_output_kw", "fuel_consumption_kg_per_s"]
    TIME_FIELD_NUMBER: _ClassVar[int]
    POWER_OUTPUT_KW_FIELD_NUMBER: _ClassVar[int]
    FUEL_CONSUMPTION_KG_PER_S_FIELD_NUMBER: _ClassVar[int]
    time: _containers.RepeatedScalarFieldContainer[float]
    power_output_kw: _containers.RepeatedScalarFieldContainer[float]
    fuel_consumption_kg_per_s: FuelConsumptionRateArray
    def __init__(
        self,
        time: _Optional[_Iterable[float]] = ...,
        power_output_kw: _Optional[_Iterable[float]] = ...,
        fuel_consumption_kg_per_s: _Optional[
            _Union[FuelConsumptionRateArray, _Mapping]
        ] = ...,
    ) -> None: ...

class GHGEmissions(_message.Message):
    __slots__ = [
        "well_to_tank",
        "tank_to_wake",
        "well_to_wake",
        "tank_to_wake_without_slip",
        "well_to_wake_without_slip",
        "tank_to_wake_from_green_fuel",
        "tank_to_wake_without_slip_from_green_fuel",
    ]
    WELL_TO_TANK_FIELD_NUMBER: _ClassVar[int]
    TANK_TO_WAKE_FIELD_NUMBER: _ClassVar[int]
    WELL_TO_WAKE_FIELD_NUMBER: _ClassVar[int]
    TANK_TO_WAKE_WITHOUT_SLIP_FIELD_NUMBER: _ClassVar[int]
    WELL_TO_WAKE_WITHOUT_SLIP_FIELD_NUMBER: _ClassVar[int]
    TANK_TO_WAKE_FROM_GREEN_FUEL_FIELD_NUMBER: _ClassVar[int]
    TANK_TO_WAKE_WITHOUT_SLIP_FROM_GREEN_FUEL_FIELD_NUMBER: _ClassVar[int]
    well_to_tank: float
    tank_to_wake: float
    well_to_wake: float
    tank_to_wake_without_slip: float
    well_to_wake_without_slip: float
    tank_to_wake_from_green_fuel: float
    tank_to_wake_without_slip_from_green_fuel: float
    def __init__(
        self,
        well_to_tank: _Optional[float] = ...,
        tank_to_wake: _Optional[float] = ...,
        well_to_wake: _Optional[float] = ...,
        tank_to_wake_without_slip: _Optional[float] = ...,
        well_to_wake_without_slip: _Optional[float] = ...,
        tank_to_wake_from_green_fuel: _Optional[float] = ...,
        tank_to_wake_without_slip_from_green_fuel: _Optional[float] = ...,
    ) -> None: ...

class ResultPerComponent(_message.Message):
    __slots__ = [
        "component_name",
        "multi_fuel_consumption_kg",
        "electric_energy_consumption_mj",
        "mechanical_energy_consumption_mj",
        "energy_stored_mj",
        "running_hours_h",
        "co2_emissions_kg",
        "nox_emissions_kg",
        "component_type",
        "rated_capacity",
        "rated_capacity_unit",
        "switchboard_id",
        "shaftline_id",
        "result_time_series",
        "fuel_consumer_type",
    ]
    COMPONENT_NAME_FIELD_NUMBER: _ClassVar[int]
    MULTI_FUEL_CONSUMPTION_KG_FIELD_NUMBER: _ClassVar[int]
    ELECTRIC_ENERGY_CONSUMPTION_MJ_FIELD_NUMBER: _ClassVar[int]
    MECHANICAL_ENERGY_CONSUMPTION_MJ_FIELD_NUMBER: _ClassVar[int]
    ENERGY_STORED_MJ_FIELD_NUMBER: _ClassVar[int]
    RUNNING_HOURS_H_FIELD_NUMBER: _ClassVar[int]
    CO2_EMISSIONS_KG_FIELD_NUMBER: _ClassVar[int]
    NOX_EMISSIONS_KG_FIELD_NUMBER: _ClassVar[int]
    COMPONENT_TYPE_FIELD_NUMBER: _ClassVar[int]
    RATED_CAPACITY_FIELD_NUMBER: _ClassVar[int]
    RATED_CAPACITY_UNIT_FIELD_NUMBER: _ClassVar[int]
    SWITCHBOARD_ID_FIELD_NUMBER: _ClassVar[int]
    SHAFTLINE_ID_FIELD_NUMBER: _ClassVar[int]
    RESULT_TIME_SERIES_FIELD_NUMBER: _ClassVar[int]
    FUEL_CONSUMER_TYPE_FIELD_NUMBER: _ClassVar[int]
    component_name: str
    multi_fuel_consumption_kg: FuelConsumptionScalar
    electric_energy_consumption_mj: float
    mechanical_energy_consumption_mj: float
    energy_stored_mj: float
    running_hours_h: float
    co2_emissions_kg: GHGEmissions
    nox_emissions_kg: float
    component_type: str
    rated_capacity: float
    rated_capacity_unit: str
    switchboard_id: int
    shaftline_id: int
    result_time_series: TimeSeriesResultForComponent
    fuel_consumer_type: str
    def __init__(
        self,
        component_name: _Optional[str] = ...,
        multi_fuel_consumption_kg: _Optional[
            _Union[FuelConsumptionScalar, _Mapping]
        ] = ...,
        electric_energy_consumption_mj: _Optional[float] = ...,
        mechanical_energy_consumption_mj: _Optional[float] = ...,
        energy_stored_mj: _Optional[float] = ...,
        running_hours_h: _Optional[float] = ...,
        co2_emissions_kg: _Optional[_Union[GHGEmissions, _Mapping]] = ...,
        nox_emissions_kg: _Optional[float] = ...,
        component_type: _Optional[str] = ...,
        rated_capacity: _Optional[float] = ...,
        rated_capacity_unit: _Optional[str] = ...,
        switchboard_id: _Optional[int] = ...,
        shaftline_id: _Optional[int] = ...,
        result_time_series: _Optional[
            _Union[TimeSeriesResultForComponent, _Mapping]
        ] = ...,
        fuel_consumer_type: _Optional[str] = ...,
    ) -> None: ...

class FeemsResult(_message.Message):
    __slots__ = [
        "duration_s",
        "multi_fuel_consumption_total_kg",
        "energy_consumption_electric_total_mj",
        "energy_consumption_mechanical_total_mj",
        "energy_stored_total_mj",
        "running_hours_main_engines_hr",
        "running_hours_genset_total_hr",
        "running_hours_fuel_cell_total_hr",
        "running_hours_pti_pto_total_hr",
        "co2_emission_total_kg",
        "nox_emission_total_kg",
        "detailed_result",
        "energy_input_mechanical_total_mj",
        "energy_input_electric_total_mj",
        "energy_consumption_propulsion_total_mj",
        "energy_consumption_auxiliary_total_mj",
    ]
    DURATION_S_FIELD_NUMBER: _ClassVar[int]
    MULTI_FUEL_CONSUMPTION_TOTAL_KG_FIELD_NUMBER: _ClassVar[int]
    ENERGY_CONSUMPTION_ELECTRIC_TOTAL_MJ_FIELD_NUMBER: _ClassVar[int]
    ENERGY_CONSUMPTION_MECHANICAL_TOTAL_MJ_FIELD_NUMBER: _ClassVar[int]
    ENERGY_STORED_TOTAL_MJ_FIELD_NUMBER: _ClassVar[int]
    RUNNING_HOURS_MAIN_ENGINES_HR_FIELD_NUMBER: _ClassVar[int]
    RUNNING_HOURS_GENSET_TOTAL_HR_FIELD_NUMBER: _ClassVar[int]
    RUNNING_HOURS_FUEL_CELL_TOTAL_HR_FIELD_NUMBER: _ClassVar[int]
    RUNNING_HOURS_PTI_PTO_TOTAL_HR_FIELD_NUMBER: _ClassVar[int]
    CO2_EMISSION_TOTAL_KG_FIELD_NUMBER: _ClassVar[int]
    NOX_EMISSION_TOTAL_KG_FIELD_NUMBER: _ClassVar[int]
    DETAILED_RESULT_FIELD_NUMBER: _ClassVar[int]
    ENERGY_INPUT_MECHANICAL_TOTAL_MJ_FIELD_NUMBER: _ClassVar[int]
    ENERGY_INPUT_ELECTRIC_TOTAL_MJ_FIELD_NUMBER: _ClassVar[int]
    ENERGY_CONSUMPTION_PROPULSION_TOTAL_MJ_FIELD_NUMBER: _ClassVar[int]
    ENERGY_CONSUMPTION_AUXILIARY_TOTAL_MJ_FIELD_NUMBER: _ClassVar[int]
    duration_s: float
    multi_fuel_consumption_total_kg: FuelConsumptionScalar
    energy_consumption_electric_total_mj: float
    energy_consumption_mechanical_total_mj: float
    energy_stored_total_mj: float
    running_hours_main_engines_hr: float
    running_hours_genset_total_hr: float
    running_hours_fuel_cell_total_hr: float
    running_hours_pti_pto_total_hr: float
    co2_emission_total_kg: GHGEmissions
    nox_emission_total_kg: float
    detailed_result: _containers.RepeatedCompositeFieldContainer[ResultPerComponent]
    energy_input_mechanical_total_mj: float
    energy_input_electric_total_mj: float
    energy_consumption_propulsion_total_mj: float
    energy_consumption_auxiliary_total_mj: float
    def __init__(
        self,
        duration_s: _Optional[float] = ...,
        multi_fuel_consumption_total_kg: _Optional[
            _Union[FuelConsumptionScalar, _Mapping]
        ] = ...,
        energy_consumption_electric_total_mj: _Optional[float] = ...,
        energy_consumption_mechanical_total_mj: _Optional[float] = ...,
        energy_stored_total_mj: _Optional[float] = ...,
        running_hours_main_engines_hr: _Optional[float] = ...,
        running_hours_genset_total_hr: _Optional[float] = ...,
        running_hours_fuel_cell_total_hr: _Optional[float] = ...,
        running_hours_pti_pto_total_hr: _Optional[float] = ...,
        co2_emission_total_kg: _Optional[_Union[GHGEmissions, _Mapping]] = ...,
        nox_emission_total_kg: _Optional[float] = ...,
        detailed_result: _Optional[
            _Iterable[_Union[ResultPerComponent, _Mapping]]
        ] = ...,
        energy_input_mechanical_total_mj: _Optional[float] = ...,
        energy_input_electric_total_mj: _Optional[float] = ...,
        energy_consumption_propulsion_total_mj: _Optional[float] = ...,
        energy_consumption_auxiliary_total_mj: _Optional[float] = ...,
    ) -> None: ...

class FeemsResultForMachinerySystem(_message.Message):
    __slots__ = ["electric_system", "mechanical_system"]
    ELECTRIC_SYSTEM_FIELD_NUMBER: _ClassVar[int]
    MECHANICAL_SYSTEM_FIELD_NUMBER: _ClassVar[int]
    electric_system: FeemsResult
    mechanical_system: FeemsResult
    def __init__(
        self,
        electric_system: _Optional[_Union[FeemsResult, _Mapping]] = ...,
        mechanical_system: _Optional[_Union[FeemsResult, _Mapping]] = ...,
    ) -> None: ...



# AUTOGENERATED! DO NOT EDIT! File to edit: ../02_Utility.ipynb.

# %% auto 0
__all__ = ["retrieve_machinery_system_from_file", "retrieve_gymir_result_from_file"]

# %% ../02_Utility.ipynb 3
import io
import os
from typing import Union

import MachSysS.system_structure_pb2 as proto_system
import MachSysS.gymir_result_pb2 as proto_gymir


def retrieve_machinery_system_from_file(
    file: Union[str, io.BytesIO],
) -> proto_system.MachinerySystem:
    if isinstance(file, str):
        file = open(file, "rb")
    system = proto_system.MachinerySystem()
    system.ParseFromString(file.read())
    file.close()
    return system


def retrieve_gymir_result_from_file(
    file: Union[str, io.BytesIO],
) -> proto_gymir.GymirResult:
    if isinstance(file, str):
        file = open(file, "rb")
    system = proto_gymir.GymirResult()
    system.ParseFromString(file.read())
    file.close()
    return system
